from indifferent.indifferent import _format_html


def test_html_css():
    assert (
        _format_html(
            comparison_split=[
                {"base": 0, "content": True, "revision": 0, "value": "a"},
                {"base": 1, "content": False, "revision": 1, "value": " "},
                {"base": 2, "content": True, "revision": None, "value": "big"},
                {"base": None, "content": True, "revision": 2, "value": "tabby"},
                {"base": 3, "content": False, "revision": 3, "value": " "},
                {"base": 4, "content": True, "revision": 4, "value": "cat"},
            ]
        )["css"]
        == """    .indifferent {
        max-width: 900px;
        min-width: 800px;
        margin: 0 auto;
        background-color: #FFF;
        padding: 30px 20px;
        color: #333;
    }

    .indifferent h2.title {
        text-align: center;
    }

    .indifferent h2.title .vs {
        font-size: 70%;
        color: #333;
    }

    .indifferent h3.subtitle {
        border-bottom: none;
        text-align: center;
        color: #555;
    }

    .indifferent .nav-links table {
        margin: 40px auto 0px;
    }

    .indifferent .nav-links td {
        border-right: 1px #888 solid;
    }

    .indifferent .nav-links td.last {
        border-right: 0px;
    }

    .indifferent .nav-links a {
        color: #333;
        padding: 5px 10px;
        text-decoration: none;
    }

    .indifferent .section h3 {
        padding-bottom: 10px;
        margin: 20px 0px 0px;
        border-bottom: 1px solid grey;
    }

    .indifferent .section {
        padding: 20px 0px;
    }

    .indifferent .detail {
        margin: 10px;
        padding: 15px;
        border: 1px solid #DDD;
        font-family: monospace;
    }

    .indifferent .matched {
        background-color: #DEDEDE;
        margin: 0px 1px;
    }

    .indifferent .added {
        color: green;
        text-decoration: underline;
        font-weight: bold;
        margin: 0px 1px
    }

    .indifferent .deleted {
        color: red;
        text-decoration: line-through;
        margin: 0px 1px;
    }
"""
    )


def test_html_html():
    assert (
        (
            _format_html(
                comparison_split=[
                    {"base": 0, "content": True, "revision": 0, "value": "a"},
                    {"base": 1, "content": False, "revision": 1, "value": " "},
                    {"base": 2, "content": True, "revision": None, "value": "big"},
                    {"base": None, "content": True, "revision": 2, "value": "tabby"},
                    {"base": 3, "content": False, "revision": 3, "value": " "},
                    {"base": 4, "content": True, "revision": 4, "value": "cat"},
                ]
            )["html"]
        )
        == """<!-- Comparison generated by Indifferent: https://github.com/brianwarner/indifferent -->
<div class="indifferent">
    <h2 class="title">Base<br /><span class="vs">vs.</span><br />Revision</h2>
    <h3 class="subtitle">50% match</h3>
    <div class="nav-links">
        <table>
            <tr>
                <td><a href="#indifferent.base">Base</a></td>
                <td><a href="#indifferent.revision">Revision</a></td>
                <td class="last"><a href="#indifferent.match">Comparison</a></td>
            </tr>
        </table>
    </div>
    <div class="section base">
        <a id="indifferent.base"></a>
        <h3>Base</h3>
        <div class="summary">
            <ul>
                <li><strong>Base length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the revision:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the revision</li>
            </ul>
        </div>
        <div class="detail">
            a big cat
        </div>
    </div>
    <div class="section revision">
        <a id="indifferent.revision"></a>
        <h3>Revision: Revision</h3>
        <div class="summary">
            <ul>
                <li><strong>Revision length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the base:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the base</li>
            </ul>
        </div>
        <div class="detail">
            a tabby cat
        </div>
    </div>
    <div class="section match">
        <a id="indifferent.match"></a>
        <h3>Comparison: 50% match</h3>
        <div class="summary">
            <ul>
                <li><strong>Identical in base and revision:</strong> 2 words and 2 separators (4 total)</li>
                <li><strong>Removed from the base:</strong> 1 word and 0 separators (1 total)</li>
                <li><strong>Added by the revision:</strong> 1 word and 0 separators (1 total)</li>
        </div>
        <div class="detail">
            <span class="matched">a</span><span class="matched"> </span><span class="deleted">big</span><span class="added">tabby</span><span class="matched"> </span><span class="matched">cat</span>
        </div>
    </div>
</div>"""  # noqa E501
    )


def test_html_inline_html():
    assert (
        _format_html(
            comparison_split=[
                {"base": 0, "content": True, "revision": 0, "value": "a"},
                {"base": 1, "content": False, "revision": 1, "value": " "},
                {"base": 2, "content": True, "revision": None, "value": "big"},
                {"base": None, "content": True, "revision": 2, "value": "tabby"},
                {"base": 3, "content": False, "revision": 3, "value": " "},
                {"base": 4, "content": True, "revision": 4, "value": "cat"},
            ],
            inline=True,
        )["html"]
        == """<!-- Comparison generated by Indifferent: https://github.com/brianwarner/indifferent -->
<div class="indifferent" style="max-width: 900px; min-width: 800px; margin: 0 auto; background-color: #FFF; padding: 30px 20px; color: #333;">
    <h2 style="text-align: center;" class="title">Base<br /><span class="vs" style="font-size: 70%; color: #333;">vs.</span><br />Revision</h2>
    <h3 style="border-bottom: none; text-align: center; color: #555;" class="subtitle">50% match</h3>
    <div class="nav-links">
        <table style="margin: 40px auto 0px;">
            <tr>
                <td style="border-right: 1px #888 solid;"><a href="#indifferent.base" style="color: #333; padding: 5px 10px; text-decoration: none;">Base</a></td>
                <td style="border-right: 1px #888 solid;"><a href="#indifferent.revision" style="color: #333; padding: 5px 10px; text-decoration: none;">Revision</a></td>
                <td style="border-right: 0px;" class="last"><a href="#indifferent.match" style="color: #333; padding: 5px 10px; text-decoration: none;">Comparison</a></td>
            </tr>
        </table>
    </div>
    <div class="section base" style="padding: 20px 0px;">
        <a id="indifferent.base"></a>
        <h3 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Base</h3>
        <div class="summary">
            <ul>
                <li><strong>Base length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the revision:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the revision</li>
            </ul>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            a big cat
        </div>
    </div>
    <div class="section revision" style="padding: 20px 0px;">
        <a id="indifferent.revision"></a>
        <h3 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Revision: Revision</h3>
        <div class="summary">
            <ul>
                <li><strong>Revision length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the base:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the base</li>
            </ul>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            a tabby cat
        </div>
    </div>
    <div class="section match" style="padding: 20px 0px;">
        <a id="indifferent.match"></a>
        <h3 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Comparison: 50% match</h3>
        <div class="summary">
            <ul>
                <li><strong>Identical in base and revision:</strong> 2 words and 2 separators (4 total)</li>
                <li><strong>Removed from the base:</strong> 1 word and 0 separators (1 total)</li>
                <li><strong>Added by the revision:</strong> 1 word and 0 separators (1 total)</li>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            <span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;">a</span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;"> </span><span class="deleted" style="color: red; text-decoration: line-through; margin: 0px 1px;">big</span><span class="added" style="color: green; text-decoration: underline; font-weight: bold; margin: 0px 1px">tabby</span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;"> </span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;">cat</span>
        </div>
    </div>
</div>"""  # noqa E501
    )


def test_html_page_css():
    assert (
        (
            _format_html(
                comparison_split=[
                    {"base": 0, "content": True, "revision": 0, "value": "a"},
                    {"base": 1, "content": False, "revision": 1, "value": " "},
                    {"base": 2, "content": True, "revision": None, "value": "big"},
                    {"base": None, "content": True, "revision": 2, "value": "tabby"},
                    {"base": 3, "content": False, "revision": 3, "value": " "},
                    {"base": 4, "content": True, "revision": 4, "value": "cat"},
                ],
                page=True,
            )["css"]
        )
        == """   .page {
        background-color: #DDD;
        font-family: sans-serif;
    }

    .indifferent {
        max-width: 900px;
        min-width: 800px;
        margin: 0 auto;
        background-color: #FFF;
        padding: 30px 20px;
        color: #333;
    }

    .indifferent h1.title {
        text-align: center;
    }

    .indifferent h1.title .vs {
        font-size: 70%;
        color: #333;
    }

    .indifferent h2.subtitle {
        border-bottom: none;
        text-align: center;
        color: #555;
    }

    .indifferent .nav-links table {
        margin: 40px auto 0px;
    }

    .indifferent .nav-links td {
        border-right: 1px #888 solid;
    }

    .indifferent .nav-links td.last {
        border-right: 0px;
    }

    .indifferent .nav-links a {
        color: #333;
        padding: 5px 10px;
        text-decoration: none;
    }

    .indifferent .section h2 {
        padding-bottom: 10px;
        margin: 20px 0px 0px;
        border-bottom: 1px solid grey;
    }

    .indifferent .section {
        padding: 20px 0px;
    }

    .indifferent .detail {
        margin: 10px;
        padding: 15px;
        border: 1px solid #DDD;
        font-family: monospace;
    }

    .indifferent .matched {
        background-color: #DEDEDE;
        margin: 0px 1px;
    }

    .indifferent .added {
        color: green;
        text-decoration: underline;
        font-weight: bold;
        margin: 0px 1px
    }

    .indifferent .deleted {
        color: red;
        text-decoration: line-through;
        margin: 0px 1px;
    }
"""
    )


def test_html_page_html():
    assert (
        (
            _format_html(
                comparison_split=[
                    {"base": 0, "content": True, "revision": 0, "value": "a"},
                    {"base": 1, "content": False, "revision": 1, "value": " "},
                    {"base": 2, "content": True, "revision": None, "value": "big"},
                    {"base": None, "content": True, "revision": 2, "value": "tabby"},
                    {"base": 3, "content": False, "revision": 3, "value": " "},
                    {"base": 4, "content": True, "revision": 4, "value": "cat"},
                ],
                page=True,
            )["html"]
        )
        == """<!doctype html>
<html lang="en-US">
<head>
<meta charset="utf-8" />
<title>Comparison of Base and Revision</title>
<link rel="stylesheet" href="indifferent.css">
</head>
<body class="page">
<!-- Comparison generated by Indifferent: https://github.com/brianwarner/indifferent -->
<div class="indifferent">
    <h1 class="title">Base<br /><span class="vs">vs.</span><br />Revision</h1>
    <h2 class="subtitle">50% match</h2>
    <div class="nav-links">
        <table>
            <tr>
                <td><a href="#indifferent.base">Base</a></td>
                <td><a href="#indifferent.revision">Revision</a></td>
                <td class="last"><a href="#indifferent.match">Comparison</a></td>
            </tr>
        </table>
    </div>
    <div class="section base">
        <a id="indifferent.base"></a>
        <h2>Base</h2>
        <div class="summary">
            <ul>
                <li><strong>Base length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the revision:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the revision</li>
            </ul>
        </div>
        <div class="detail">
            a big cat
        </div>
    </div>
    <div class="section revision">
        <a id="indifferent.revision"></a>
        <h2>Revision: Revision</h2>
        <div class="summary">
            <ul>
                <li><strong>Revision length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the base:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the base</li>
            </ul>
        </div>
        <div class="detail">
            a tabby cat
        </div>
    </div>
    <div class="section match">
        <a id="indifferent.match"></a>
        <h2>Comparison: 50% match</h2>
        <div class="summary">
            <ul>
                <li><strong>Identical in base and revision:</strong> 2 words and 2 separators (4 total)</li>
                <li><strong>Removed from the base:</strong> 1 word and 0 separators (1 total)</li>
                <li><strong>Added by the revision:</strong> 1 word and 0 separators (1 total)</li>
        </div>
        <div class="detail">
            <span class="matched">a</span><span class="matched"> </span><span class="deleted">big</span><span class="added">tabby</span><span class="matched"> </span><span class="matched">cat</span>
        </div>
    </div>
</div>
</body>
</html>"""  # noqa E501
    )


def test_html_page_internal_html():
    assert (
        (
            _format_html(
                comparison_split=[
                    {"base": 0, "content": True, "revision": 0, "value": "a"},
                    {"base": 1, "content": False, "revision": 1, "value": " "},
                    {"base": 2, "content": True, "revision": None, "value": "big"},
                    {"base": None, "content": True, "revision": 2, "value": "tabby"},
                    {"base": 3, "content": False, "revision": 3, "value": " "},
                    {"base": 4, "content": True, "revision": 4, "value": "cat"},
                ],
                page=True,
                internal=True,
            )["html"]
        )
        == """<!doctype html>
<html lang="en-US">
<head>
<meta charset="utf-8" />
<title>Comparison of Base and Revision</title>
<style>
   .page {
        background-color: #DDD;
        font-family: sans-serif;
    }

    .indifferent {
        max-width: 900px;
        min-width: 800px;
        margin: 0 auto;
        background-color: #FFF;
        padding: 30px 20px;
        color: #333;
    }

    .indifferent h1.title {
        text-align: center;
    }

    .indifferent h1.title .vs {
        font-size: 70%;
        color: #333;
    }

    .indifferent h2.subtitle {
        border-bottom: none;
        text-align: center;
        color: #555;
    }

    .indifferent .nav-links table {
        margin: 40px auto 0px;
    }

    .indifferent .nav-links td {
        border-right: 1px #888 solid;
    }

    .indifferent .nav-links td.last {
        border-right: 0px;
    }

    .indifferent .nav-links a {
        color: #333;
        padding: 5px 10px;
        text-decoration: none;
    }

    .indifferent .section h2 {
        padding-bottom: 10px;
        margin: 20px 0px 0px;
        border-bottom: 1px solid grey;
    }

    .indifferent .section {
        padding: 20px 0px;
    }

    .indifferent .detail {
        margin: 10px;
        padding: 15px;
        border: 1px solid #DDD;
        font-family: monospace;
    }

    .indifferent .matched {
        background-color: #DEDEDE;
        margin: 0px 1px;
    }

    .indifferent .added {
        color: green;
        text-decoration: underline;
        font-weight: bold;
        margin: 0px 1px
    }

    .indifferent .deleted {
        color: red;
        text-decoration: line-through;
        margin: 0px 1px;
    }
</style>
</head>
<body class="page">
<!-- Comparison generated by Indifferent: https://github.com/brianwarner/indifferent -->
<div class="indifferent">
    <h1 class="title">Base<br /><span class="vs">vs.</span><br />Revision</h1>
    <h2 class="subtitle">50% match</h2>
    <div class="nav-links">
        <table>
            <tr>
                <td><a href="#indifferent.base">Base</a></td>
                <td><a href="#indifferent.revision">Revision</a></td>
                <td class="last"><a href="#indifferent.match">Comparison</a></td>
            </tr>
        </table>
    </div>
    <div class="section base">
        <a id="indifferent.base"></a>
        <h2>Base</h2>
        <div class="summary">
            <ul>
                <li><strong>Base length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the revision:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the revision</li>
            </ul>
        </div>
        <div class="detail">
            a big cat
        </div>
    </div>
    <div class="section revision">
        <a id="indifferent.revision"></a>
        <h2>Revision: Revision</h2>
        <div class="summary">
            <ul>
                <li><strong>Revision length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the base:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the base</li>
            </ul>
        </div>
        <div class="detail">
            a tabby cat
        </div>
    </div>
    <div class="section match">
        <a id="indifferent.match"></a>
        <h2>Comparison: 50% match</h2>
        <div class="summary">
            <ul>
                <li><strong>Identical in base and revision:</strong> 2 words and 2 separators (4 total)</li>
                <li><strong>Removed from the base:</strong> 1 word and 0 separators (1 total)</li>
                <li><strong>Added by the revision:</strong> 1 word and 0 separators (1 total)</li>
        </div>
        <div class="detail">
            <span class="matched">a</span><span class="matched"> </span><span class="deleted">big</span><span class="added">tabby</span><span class="matched"> </span><span class="matched">cat</span>
        </div>
    </div>
</div>
</body>
</html>"""  # noqa E501
    )


def test_html_page_inline_html():
    assert (
        (
            _format_html(
                comparison_split=[
                    {"base": 0, "content": True, "revision": 0, "value": "a"},
                    {"base": 1, "content": False, "revision": 1, "value": " "},
                    {"base": 2, "content": True, "revision": None, "value": "big"},
                    {"base": None, "content": True, "revision": 2, "value": "tabby"},
                    {"base": 3, "content": False, "revision": 3, "value": " "},
                    {"base": 4, "content": True, "revision": 4, "value": "cat"},
                ],
                page=True,
                inline=True,
            )["html"]
        )
        == """<!doctype html>
<html lang="en-US">
<head>
<meta charset="utf-8" />
<title>Comparison of Base and Revision</title>
</head>
<body class="page" style="background-color: #DDD; font-family: sans-serif;">
<!-- Comparison generated by Indifferent: https://github.com/brianwarner/indifferent -->
<div class="indifferent" style="max-width: 900px; min-width: 800px; margin: 0 auto; background-color: #FFF; padding: 30px 20px; color: #333;">
    <h1 style="text-align: center;" class="title">Base<br /><span class="vs" style="font-size: 70%; color: #333;">vs.</span><br />Revision</h1>
    <h2 style="border-bottom: none; text-align: center; color: #555;" class="subtitle">50% match</h2>
    <div class="nav-links">
        <table style="margin: 40px auto 0px;">
            <tr>
                <td style="border-right: 1px #888 solid;"><a href="#indifferent.base" style="color: #333; padding: 5px 10px; text-decoration: none;">Base</a></td>
                <td style="border-right: 1px #888 solid;"><a href="#indifferent.revision" style="color: #333; padding: 5px 10px; text-decoration: none;">Revision</a></td>
                <td style="border-right: 0px;" class="last"><a href="#indifferent.match" style="color: #333; padding: 5px 10px; text-decoration: none;">Comparison</a></td>
            </tr>
        </table>
    </div>
    <div class="section base" style="padding: 20px 0px;">
        <a id="indifferent.base"></a>
        <h2 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Base</h2>
        <div class="summary">
            <ul>
                <li><strong>Base length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the revision:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the revision</li>
            </ul>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            a big cat
        </div>
    </div>
    <div class="section revision" style="padding: 20px 0px;">
        <a id="indifferent.revision"></a>
        <h2 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Revision: Revision</h2>
        <div class="summary">
            <ul>
                <li><strong>Revision length:</strong> 3 words and 2 separators (5 total)</li>
                <li><strong>Words also in the base:</strong> 2 of 3 (67%)</li>
                <li><strong>Similarity:</strong> 80% identical to the base</li>
            </ul>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            a tabby cat
        </div>
    </div>
    <div class="section match" style="padding: 20px 0px;">
        <a id="indifferent.match"></a>
        <h2 style="padding-bottom: 10px; margin: 20px 0px 0px; border-bottom: 1px solid grey;">Comparison: 50% match</h2>
        <div class="summary">
            <ul>
                <li><strong>Identical in base and revision:</strong> 2 words and 2 separators (4 total)</li>
                <li><strong>Removed from the base:</strong> 1 word and 0 separators (1 total)</li>
                <li><strong>Added by the revision:</strong> 1 word and 0 separators (1 total)</li>
        </div>
        <div class="detail" style="margin: 10px; padding: 15px; border: 1px solid #DDD; font-family: monospace;">
            <span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;">a</span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;"> </span><span class="deleted" style="color: red; text-decoration: line-through; margin: 0px 1px;">big</span><span class="added" style="color: green; text-decoration: underline; font-weight: bold; margin: 0px 1px">tabby</span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;"> </span><span class="matched" style="background-color: #DEDEDE; margin: 0px 1px;">cat</span>
        </div>
    </div>
</div>
</body>
</html>"""  # noqa E501
    )
